import SortableTree, { map, getDescendantCount, walk, getNodeAtPath, changeNodeAtPath, getFlatDataFromTree } from 'atroo-react-sortable-tree';
export { addNodeUnderParent, changeNodeAtPath, find, getFlatDataFromTree, getNodeAtPath, insertNode, removeNode, removeNodeAtPath, walk } from 'atroo-react-sortable-tree';
import shortid from 'shortid';
import React, { useState, useEffect, Children, isValidElement, cloneElement, useRef, useCallback, useMemo, useImperativeHandle } from 'react';
import 'atroo-react-sortable-tree/style.css';
import InfiniteLoader from 'react-window-infinite-loader';
import axios from 'axios';

function _extends() {
  _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  return _extends.apply(this, arguments);
}

function _objectWithoutPropertiesLoose(source, excluded) {
  if (source == null) return {};
  var target = {};
  var sourceKeys = Object.keys(source);
  var key, i;

  for (i = 0; i < sourceKeys.length; i++) {
    key = sourceKeys[i];
    if (excluded.indexOf(key) >= 0) continue;
    target[key] = source[key];
  }

  return target;
}

function _unsupportedIterableToArray(o, minLen) {
  if (!o) return;
  if (typeof o === "string") return _arrayLikeToArray(o, minLen);
  var n = Object.prototype.toString.call(o).slice(8, -1);
  if (n === "Object" && o.constructor) n = o.constructor.name;
  if (n === "Map" || n === "Set") return Array.from(o);
  if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen);
}

function _arrayLikeToArray(arr, len) {
  if (len == null || len > arr.length) len = arr.length;

  for (var i = 0, arr2 = new Array(len); i < len; i++) arr2[i] = arr[i];

  return arr2;
}

function _createForOfIteratorHelperLoose(o, allowArrayLike) {
  var it = typeof Symbol !== "undefined" && o[Symbol.iterator] || o["@@iterator"];
  if (it) return (it = it.call(o)).next.bind(it);

  if (Array.isArray(o) || (it = _unsupportedIterableToArray(o)) || allowArrayLike && o && typeof o.length === "number") {
    if (it) o = it;
    var i = 0;
    return function () {
      if (i >= o.length) return {
        done: true
      };
      return {
        done: false,
        value: o[i++]
      };
    };
  }

  throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.");
}

var appendNodes = function appendNodes(oldTree, newTree, place) {
  var treeIndex = 0;

  if (place === "start") {
    return {
      treeIndex: treeIndex,
      treeData: [].concat(newTree, oldTree)
    };
  }

  if (place === "end") {
    treeIndex = oldTree.length;
    return {
      treeIndex: treeIndex,
      treeData: [].concat(oldTree, newTree)
    };
  }

  if (place === "instead") {
    return {
      treeIndex: treeIndex,
      treeData: newTree
    };
  }

  return {
    treeIndex: treeIndex,
    treeData: oldTree
  };
};

var replaceNodes = function replaceNodes(oldTree, newTree, startIndex) {
  var start = 0;

  if (Array.isArray(startIndex)) {
    start = startIndex[0];
  } else {
    start = startIndex;
  }

  var end = start + newTree.length - 1;
  oldTree.forEach(function (_, idx) {
    var s = idx - start;
    var e = idx - end;

    if (s < 0 || e > 0) {
      return;
    }

    var n = newTree[s];

    if (n) {
      oldTree[idx] = n;
    }
  });
  return {
    treeIndex: startIndex,
    treeData: oldTree
  };
};

var appendOrReplace = function appendOrReplace(oldTree, newTree, _appendOrReplace) {
  if (typeof _appendOrReplace === "string") {
    return appendNodes(oldTree, newTree, _appendOrReplace);
  }

  if (typeof (_appendOrReplace === null || _appendOrReplace === void 0 ? void 0 : _appendOrReplace.start) === "number") {
    return replaceNodes(oldTree, newTree, _appendOrReplace === null || _appendOrReplace === void 0 ? void 0 : _appendOrReplace.start);
  }

  return {
    treeData: oldTree,
    treeIndex: 0
  };
};

var addNodesUnderParent = function addNodesUnderParent(_ref) {
  var treeData = _ref.treeData,
      newNodes = _ref.newNodes,
      _ref$parentKey = _ref.parentKey,
      parentKey = _ref$parentKey === void 0 ? null : _ref$parentKey,
      getNodeKey = _ref.getNodeKey,
      _ref$ignoreCollapsed = _ref.ignoreCollapsed,
      ignoreCollapsed = _ref$ignoreCollapsed === void 0 ? true : _ref$ignoreCollapsed,
      _ref$expandParent = _ref.expandParent,
      expandParent = _ref$expandParent === void 0 ? false : _ref$expandParent,
      replace = _ref.replace,
      append = _ref.append,
      parentData = _ref.parentData;

  if (parentKey === null) {
    var _changedTreeData = map({
      treeData: treeData,
      getNodeKey: getNodeKey,
      ignoreCollapsed: ignoreCollapsed,
      callback: function callback(_ref2) {
        var node = _ref2.node,
            path = _ref2.path;
        var key = path ? path[path.length - 1] : null;

        if (key === parentKey) {
          return _extends({}, node, parentData);
        }

        return node;
      }
    });

    return appendOrReplace(_changedTreeData, newNodes, append || replace);
  }

  var insertedTreeIndex = null;
  var hasBeenAdded = false;
  var changedTreeData = map({
    treeData: treeData,
    getNodeKey: getNodeKey,
    ignoreCollapsed: ignoreCollapsed,
    callback: function callback(_ref3) {
      var node = _ref3.node,
          treeIndex = _ref3.treeIndex,
          path = _ref3.path;
      var key = path ? path[path.length - 1] : null;

      if (hasBeenAdded || key !== parentKey) {
        return node;
      }

      hasBeenAdded = true;

      var parentNode = _extends({}, node, parentData);

      if (expandParent) {
        parentNode.expanded = true;
      }

      if (!parentNode.children) {
        insertedTreeIndex = treeIndex + 1;
        return _extends({}, parentNode, {
          children: [].concat(newNodes)
        });
      }

      if (typeof parentNode.children === "function") {
        throw new Error("Cannot add to children defined by a function");
      }

      var parentNodeChildren = parentNode.children;
      var nextTreeIndex = treeIndex + 1;

      for (var i = 0; i < parentNodeChildren.length; i += 1) {
        nextTreeIndex += 1 + getDescendantCount({
          node: parentNodeChildren[i],
          ignoreCollapsed: ignoreCollapsed
        });
      }

      insertedTreeIndex = nextTreeIndex;
      var children = appendOrReplace(parentNodeChildren, newNodes, append || replace).treeData;
      return _extends({}, parentNode, {
        children: children
      });
    }
  });

  if (!hasBeenAdded) {
    throw new Error("No node found with the given key.");
  }

  return {
    treeData: changedTreeData,
    treeIndex: insertedTreeIndex
  };
};

var getNodeKey = function getNodeKey(_ref) {
  var node = _ref.node;
  return node.id;
};

var makeDefaultDataNode = function makeDefaultDataNode(data) {
  var node = {
    data: data,
    id: data.media_id,
    title: data.title,
    childrenCount: data.countByAssetType.collection
  };
  return node;
};

var PLACEHOLDER_TYPE = "placeholder";
var PLACEHOLDER_LOADING_TYPE = "placeholder_loading";
var PLACEHOLDER_SKELETON_TYPE = "placeholder_skeleton";
var LOADING_BATCH_COUNT = 10;

var findParentNode = function findParentNode(parentId, treeData) {
  if (!parentId) {
    return null;
  }

  var parentKey = null;
  var newNode = null;

  var walkCallback = function walkCallback(info) {
    if (info.node.id === parentId) {
      newNode = info.node;
      parentKey = info.path ? info.path[info.path.length - 1] : null;
      return false;
    }

    return;
  };

  walk({
    treeData: treeData,
    getNodeKey: getNodeKey,
    callback: walkCallback,
    ignoreCollapsed: false
  });

  if (!newNode || !parentKey) {
    return null;
  }

  return {
    parentNode: newNode,
    parentKey: parentKey
  };
};

var findParentAndAppendNodes = function findParentAndAppendNodes(params) {
  var parentId = params.parentId,
      treeData = params.treeData,
      nodesToAppend = params.nodesToAppend,
      append = params.append,
      replace = params.replace,
      parentData = params.parentData;
  var parentNodeBag = findParentNode(parentId, treeData);
  var parentKey = null;
  var newNode = null;

  if (parentNodeBag) {
    parentKey = parentNodeBag.parentKey;
    newNode = parentNodeBag.parentNode;
  }

  var expanded = params.parentExpanded;

  if (expanded === undefined) {
    expanded = newNode ? newNode.expanded : false;
  }

  try {
    var newTree = addNodesUnderParent({
      treeData: treeData,
      newNodes: nodesToAppend,
      getNodeKey: getNodeKey,
      parentKey: parentKey,
      expandParent: expanded,
      ignoreCollapsed: false,
      append: append,
      replace: replace,
      parentData: parentData
    });
    return newTree.treeData;
  } catch (error) {
    console.error(error);
    throw error;
  }
};

var buildAndAppendSkeletons = function buildAndAppendSkeletons(params) {
  var parent = params.parent,
      count = params.count,
      addJob = params.addJob,
      setTreeData = params.setTreeData,
      newNodesPromise = params.newNodesPromise,
      append = params.append,
      parentData = params.parentData,
      skeletonRenderer = params.skeletonRenderer;
  var skeletons = Array(count).fill(null).map(function () {
    var id = shortid.generate();
    return {
      type: PLACEHOLDER_TYPE,
      placeholderType: PLACEHOLDER_SKELETON_TYPE,
      title: skeletonRenderer ? skeletonRenderer() : "[skeleton...]: " + id,
      id: id,
      data: {
        parent: parent
      }
    };
  });
  var newNodes = null;
  newNodesPromise === null || newNodesPromise === void 0 ? void 0 : newNodesPromise.then(function (n) {
    newNodes = n;
  });
  return addJob(function () {
    if (newNodes) {
      return;
    }

    setTreeData(function (oldTree) {
      try {
        var parentId = parent ? parent.id : null;
        return findParentAndAppendNodes({
          parentId: parentId,
          treeData: oldTree,
          nodesToAppend: skeletons,
          append: append,
          parentData: parentData
        });
      } catch (err) {
        return oldTree;
      }
    });
  });
};

var createLoader = function createLoader(loadData, makeNode) {
  return function (parent, startIndex, endIndex) {
    try {
      return Promise.resolve(loadData(parent, startIndex, endIndex)).then(function (data) {
        return data.map(function (d) {
          return makeNode(d);
        });
      });
    } catch (e) {
      return Promise.reject(e);
    }
  };
};

var createOnVisibilityToggle = function createOnVisibilityToggle(fetchNodes) {
  return function (data) {
    try {
      if (!data.expanded) {
        return Promise.resolve();
      }

      var node = data.node;
      var endIndex = Math.min(node.childrenCount - 1, 10);
      fetchNodes({
        parentNode: node,
        startIndex: 0,
        endIndex: endIndex
      }, node.childrenCount, false, true);
      return Promise.resolve();
    } catch (e) {
      return Promise.reject(e);
    }
  };
};

var createCannotDragSkeleton = function createCannotDragSkeleton(dragForbiddenRef) {
  return function (data) {
    if (dragForbiddenRef.current) {
      return false;
    }

    var node = data.node;

    if (node.type === PLACEHOLDER_TYPE) {
      return false;
    }

    return true;
  };
};

var cannotDropSkeleton = function cannotDropSkeleton(data) {
  var node = data.node;

  if (node.type === PLACEHOLDER_TYPE) {
    return false;
  }

  return true;
};

var skeletonCannotHaveChildren = function skeletonCannotHaveChildren(node) {
  if (node.type === PLACEHOLDER_TYPE) {
    return false;
  }

  return true;
};

var useOnDragFinished = function useOnDragFinished(treeData, onDragFinished) {
  var _useState = useState(null),
      draggedNode = _useState[0],
      setDraggedNode = _useState[1];

  var _useState2 = useState(false),
      isDragging = _useState2[0],
      setIsDragging = _useState2[1];

  var _useState3 = useState(null),
      oldData = _useState3[0],
      setOldData = _useState3[1];

  useEffect(function () {
    if (!onDragFinished || !isDragging || !draggedNode) {
      return;
    }

    var walkCallback = function walkCallback(info) {
      var node = info.node,
          parentNode = info.parentNode,
          path = info.path;

      if (node.id !== draggedNode.id) {
        return;
      }

      setOldData({
        parentNode: parentNode,
        path: path
      });
      return false;
    };

    walk({
      treeData: treeData,
      getNodeKey: getNodeKey,
      callback: walkCallback,
      ignoreCollapsed: false
    });
  }, [isDragging, draggedNode]);
  useEffect(function () {
    if (isDragging || !draggedNode || !onDragFinished) {
      return;
    }

    var walkCallback = function walkCallback(info) {
      var node = info.node,
          parentNode = info.parentNode,
          path = info.path;

      if (node.id !== draggedNode.id) {
        return;
      }

      if (!oldData) {
        console.error("oldParentNode is not found");
        return false;
      }

      onDragFinished({
        parentNode: parentNode,
        node: node,
        path: path,
        oldParentNode: oldData.parentNode,
        oldPath: oldData.path,
        treeData: treeData
      });
      return false;
    };

    walk({
      treeData: treeData,
      getNodeKey: getNodeKey,
      callback: walkCallback,
      ignoreCollapsed: false
    });
    setDraggedNode(null);
    setOldData(null);
  }, [treeData, isDragging, draggedNode]);

  var onDragStateChanged = function onDragStateChanged(_ref) {
    var isDragging = _ref.isDragging,
        draggedNode = _ref.draggedNode;

    if (!onDragFinished) {
      return;
    }

    if (draggedNode) {
      setDraggedNode(draggedNode);
    }

    setIsDragging(isDragging);
  };

  return onDragStateChanged;
};

var styles = {"rowWrapper":"_1UiuL","rowWrapperDragDisabled":"_3GYGD","row":"_3sSEL","rowLandingPad":"_dBblz","rowCancelPad":"_2A3f-","rowSearchMatch":"_ayubi","rowSearchFocus":"_2AeF8","rowContents":"_1QaiZ","rowLabel":"_mmiO8","rowToolbar":"_10grV","toolbarButton":"_c31HW","collapseButton":"_34t-H","expandButton":"_yurgk","expandIcon":"_1vyNP","expandIcon--collapse":"_28ZyH","lineChildren":"_2XjjU","lineBlock":"_zlaj1","absoluteLineBlock":"_1xABi","highlightLineVertical":"_1yHdD","arrow-pulse":"_2zxjq","highlightTopLeftCorner":"_1AbUi","highlightBottomLeftCorner":"_2OIbO","folder":"_2pVjA","childrenCount":"_3Du_q"};

var isPlaceholder = function isPlaceholder(node) {
  return node.type === PLACEHOLDER_TYPE;
};

var _excluded = ["scaffoldBlockPxWidth", "toggleChildrenVisibility", "connectDragPreview", "connectDragSource", "isDragging", "canDrop", "canDrag", "node", "title", "draggedNode", "path", "treeIndex", "isSearchMatch", "isSearchFocus", "icons", "buttons", "className", "style", "didDrop", "lowerSiblingCounts", "listIndex", "swapFrom", "swapLength", "swapDepth", "treeId", "isOver", "parentNode", "rowDirection"];

function isDescendant(older, younger) {
  return !!older.children && typeof older.children !== "function" && older.children.some(function (child) {
    return child === younger || isDescendant(child, younger);
  });
}

var createNodeContentRenderer = function createNodeContentRenderer(renderers) {
  var renderSkeletons = renderers === null || renderers === void 0 ? void 0 : renderers.skeletons;
  var renderNode = renderers === null || renderers === void 0 ? void 0 : renderers.node;

  var NodeContentRenderer = function NodeContentRenderer(props) {
    var scaffoldBlockPxWidth = props.scaffoldBlockPxWidth,
        toggleChildrenVisibility = props.toggleChildrenVisibility,
        connectDragPreview = props.connectDragPreview,
        connectDragSource = props.connectDragSource,
        canDrop = props.canDrop,
        canDrag = props.canDrag,
        node = props.node,
        title = props.title,
        draggedNode = props.draggedNode,
        path = props.path,
        treeIndex = props.treeIndex,
        isSearchMatch = props.isSearchMatch,
        isSearchFocus = props.isSearchFocus,
        _props$icons = props.icons,
        icons = _props$icons === void 0 ? [] : _props$icons,
        _props$buttons = props.buttons,
        buttons = _props$buttons === void 0 ? [] : _props$buttons,
        className = props.className,
        style = props.style,
        lowerSiblingCounts = props.lowerSiblingCounts,
        listIndex = props.listIndex,
        swapFrom = props.swapFrom,
        swapLength = props.swapLength,
        swapDepth = props.swapDepth,
        isOver = props.isOver,
        parentNode = props.parentNode,
        otherProps = _objectWithoutPropertiesLoose(props, _excluded);

    var nodeTitle = title || node.title;
    var isDraggedDescendant = draggedNode && isDescendant(draggedNode, node);
    var isLandingPadActive = isOver;
    var scaffold = [];
    lowerSiblingCounts.forEach(function (_, i) {
      scaffold.push(React.createElement("div", {
        key: "pre_" + (1 + i),
        style: {
          width: scaffoldBlockPxWidth
        },
        className: styles.lineBlock
      }));

      if (treeIndex !== listIndex && i === swapDepth) {
        var highlightLineClass = "";

        if (swapFrom !== undefined && swapLength !== undefined && listIndex === swapFrom + swapLength - 1) {
          highlightLineClass = styles.highlightBottomLeftCorner;
        } else if (treeIndex === swapFrom) {
          highlightLineClass = styles.highlightTopLeftCorner;
        } else {
          highlightLineClass = styles.highlightLineVertical;
        }

        scaffold.push(React.createElement("div", {
          key: "highlight_" + (1 + i),
          style: {
            width: scaffoldBlockPxWidth,
            left: scaffoldBlockPxWidth * i
          },
          className: styles.absoluteLineBlock + " " + highlightLineClass
        }));
      }
    });
    var expandButtonLeft = Math.round((lowerSiblingCounts.length - 0.7) * scaffoldBlockPxWidth * 10) / 10 - 8.4;
    var expandNodeClassName = node.expanded ? styles.collapseButton : styles.expandButton;
    var expandButtonStyle = {
      left: expandButtonLeft
    };
    var onExpandButtonClick = toggleChildrenVisibility ? function () {
      return toggleChildrenVisibility({
        node: node,
        path: path,
        treeIndex: treeIndex
      });
    } : undefined;
    var rowWrapperClassName = styles.rowWrapper + (!canDrag ? " " + styles.rowWrapperDragDisabled : "");
    var rowClassName = styles.row + (isLandingPadActive ? " " + styles.rowLandingPad : "") + (isLandingPadActive && !canDrop ? " " + styles.rowCancelPad : "") + (isSearchMatch ? " " + styles.rowSearchMatch : "") + (isSearchFocus ? " " + styles.rowSearchFocus : "") + (className ? " " + className : "");

    var rowStyle = _extends({
      opacity: isDraggedDescendant ? 0.5 : 1
    }, style);

    var placeholder = isPlaceholder(node);
    var nTitle = typeof nodeTitle === "function" ? nodeTitle({
      node: node,
      path: path,
      treeIndex: treeIndex
    }) : nodeTitle;
    var nodeContent;

    if (renderNode) {
      var p = parentNode;
      var n = node;
      nodeContent = renderNode({
        parentNode: p,
        node: n,
        expandButtonStyle: expandButtonStyle,
        onExpandButtonClick: onExpandButtonClick,
        rowStyle: rowStyle,
        isPlaceholder: placeholder,
        nodeTitle: nTitle,
        buttons: buttons,
        icons: icons,
        connectDragPreview: connectDragPreview,
        scaffold: scaffold,
        canDrag: canDrag,
        otherProps: otherProps,
        defaultClassNames: {
          rowWrapperClassName: rowWrapperClassName,
          rowClassName: rowClassName,
          expandNodeClassName: expandNodeClassName,
          expandIcon: styles.expandIcon,
          "expandIcon--collapse": styles["expandIcon--collapse"],
          rowContents: styles.rowContents,
          rowToolbar: styles.rowToolbar,
          toolbarButton: styles.toolbarButton,
          rowLabel: styles.rowLabel,
          rowTitle: styles.rowTitle,
          childrenCount: styles.childrenCount,
          rowContentsDragDisabled: styles.rowContentsDragDisabled,
          folder: styles.folder
        }
      });
    } else {
      nodeContent = React.createElement("div", Object.assign({
        style: {
          height: "100%"
        }
      }, otherProps), toggleChildrenVisibility && node.childrenCount ? React.createElement("button", {
        type: "button",
        "aria-label": node.expanded ? "Collapse" : "Expand",
        className: expandNodeClassName,
        style: expandButtonStyle,
        onClick: onExpandButtonClick
      }, node.expanded ? "Collapse" : "Expand") : null, React.createElement("div", {
        className: rowWrapperClassName
      }, connectDragPreview(React.createElement("div", {
        style: {
          display: "flex"
        }
      }, scaffold, React.createElement("div", {
        className: rowClassName,
        style: rowStyle
      }, React.createElement("div", {
        className: styles.rowContents + (!canDrag ? " " + styles.rowContentsDragDisabled : "")
      }, placeholder && renderSkeletons ? renderSkeletons(parentNode) : React.createElement("div", null, React.createElement("div", {
        className: styles.rowToolbar
      }, icons.map(function (icon, index) {
        return React.createElement("div", {
          key: index,
          className: styles.toolbarButton
        }, icon);
      })), React.createElement("div", {
        className: styles.rowLabel
      }, React.createElement("span", {
        className: styles.rowTitle
      }, nTitle, Boolean(node.childrenCount) && React.createElement("span", {
        className: styles.childrenCount
      }, "(", node.childrenCount, ")"))), React.createElement("div", {
        className: styles.rowToolbar
      }, buttons.map(function (btn, index) {
        return React.createElement("div", {
          key: index,
          className: styles.toolbarButton
        }, btn);
      })))))))));
    }

    return canDrag ? connectDragSource(nodeContent, {
      dropEffect: "copy"
    }) : nodeContent;
  };

  return NodeContentRenderer;
};

var styles$1 = {};

var _excluded$1 = ["children", "listIndex", "swapFrom", "swapLength", "swapDepth", "scaffoldBlockPxWidth", "lowerSiblingCounts", "connectDropTarget", "isOver", "draggedNode", "canDrop", "treeIndex", "treeId", "getPrevRow", "node", "path", "rowDirection"];
var TreeNodeRenderer = function TreeNodeRenderer(props) {
  var children = props.children,
      listIndex = props.listIndex,
      swapFrom = props.swapFrom,
      swapLength = props.swapLength,
      swapDepth = props.swapDepth,
      lowerSiblingCounts = props.lowerSiblingCounts,
      connectDropTarget = props.connectDropTarget,
      isOver = props.isOver,
      draggedNode = props.draggedNode,
      canDrop = props.canDrop,
      node = props.node,
      otherProps = _objectWithoutPropertiesLoose(props, _excluded$1);

  var mapChildren = function mapChildren(children) {
    return Children.map(children, function (child) {
      if (!isValidElement(child)) {
        return child;
      }

      return cloneElement(child, {
        isOver: isOver,
        canDrop: canDrop,
        draggedNode: draggedNode,
        lowerSiblingCounts: lowerSiblingCounts,
        listIndex: listIndex,
        swapFrom: swapFrom,
        swapLength: swapLength,
        swapDepth: swapDepth
      });
    });
  };

  var nodeJsx = React.createElement("div", Object.assign({
    key: node.id
  }, otherProps, {
    className: styles$1.node
  }), mapChildren(children));
  return connectDropTarget(nodeJsx);
};

var createFileExplorerTheme = function createFileExplorerTheme(renderers) {
  return {
    nodeContentRenderer: createNodeContentRenderer(renderers),
    treeNodeRenderer: TreeNodeRenderer,
    scaffoldBlockPxWidth: 25,
    rowHeight: 25,
    slideRegionSize: 50
  };
};

var useNodeSelection = function useNodeSelection(onNodeSelected) {
  var _useState = useState(null),
      selectedNode = _useState[0],
      setSelectedNode = _useState[1];

  useEffect(function () {
    if (onNodeSelected) {
      onNodeSelected(selectedNode);
    }
  }, [selectedNode]);

  var generateNodeProps = function generateNodeProps(data) {
    return {
      onClick: function onClick(e) {
        if (e.target.closest('[type="button"]')) {
          return;
        }

        if (data.node.type === PLACEHOLDER_TYPE) {
          return;
        }

        var node = data.node;

        if (selectedNode && node.id === selectedNode.id) {
          setSelectedNode(null);
        } else {
          setSelectedNode(node);
        }
      },
      selected: data.node.id === (selectedNode === null || selectedNode === void 0 ? void 0 : selectedNode.id)
    };
  };

  return {
    generateNodeProps: generateNodeProps
  };
};

var createLoadNodes = function createLoadNodes(url) {
  return function (parent, startIndex, endIndex) {
    try {
      return Promise.resolve(axios.get(url, {
        params: {
          parent: parent,
          startIndex: startIndex,
          endIndex: endIndex
        }
      })).then(function (resp) {
        return resp.data;
      });
    } catch (e) {
      return Promise.reject(e);
    }
  };
};

// A type of promise-like that resolves synchronously and supports only one observer
const _Pact = /*#__PURE__*/(function() {
	function _Pact() {}
	_Pact.prototype.then = function(onFulfilled, onRejected) {
		const result = new _Pact();
		const state = this.s;
		if (state) {
			const callback = state & 1 ? onFulfilled : onRejected;
			if (callback) {
				try {
					_settle(result, 1, callback(this.v));
				} catch (e) {
					_settle(result, 2, e);
				}
				return result;
			} else {
				return this;
			}
		}
		this.o = function(_this) {
			try {
				const value = _this.v;
				if (_this.s & 1) {
					_settle(result, 1, onFulfilled ? onFulfilled(value) : value);
				} else if (onRejected) {
					_settle(result, 1, onRejected(value));
				} else {
					_settle(result, 2, value);
				}
			} catch (e) {
				_settle(result, 2, e);
			}
		};
		return result;
	};
	return _Pact;
})();

// Settles a pact synchronously
function _settle(pact, state, value) {
	if (!pact.s) {
		if (value instanceof _Pact) {
			if (value.s) {
				if (state & 1) {
					state = value.s;
				}
				value = value.v;
			} else {
				value.o = _settle.bind(null, pact, state);
				return;
			}
		}
		if (value && value.then) {
			value.then(_settle.bind(null, pact, state), _settle.bind(null, pact, 2));
			return;
		}
		pact.s = state;
		pact.v = value;
		const observer = pact.o;
		if (observer) {
			observer(pact);
		}
	}
}

function _isSettledPact(thenable) {
	return thenable instanceof _Pact && thenable.s & 1;
}

// Asynchronously iterate through an object that has a length property, passing the index as the first argument to the callback (even as the length property changes)
function _forTo(array, body, check) {
	var i = -1, pact, reject;
	function _cycle(result) {
		try {
			while (++i < array.length && (!check || !check())) {
				result = body(i);
				if (result && result.then) {
					if (_isSettledPact(result)) {
						result = result.v;
					} else {
						result.then(_cycle, reject || (reject = _settle.bind(null, pact = new _Pact(), 2)));
						return;
					}
				}
			}
			if (pact) {
				_settle(pact, 1, result);
			} else {
				pact = result;
			}
		} catch (e) {
			_settle(pact || (pact = new _Pact()), 2, e);
		}
	}
	_cycle();
	return pact;
}

const _iteratorSymbol = /*#__PURE__*/ typeof Symbol !== "undefined" ? (Symbol.iterator || (Symbol.iterator = Symbol("Symbol.iterator"))) : "@@iterator";

// Asynchronously iterate through an object's values
// Uses for...of if the runtime supports it, otherwise iterates until length on a copy
function _forOf(target, body, check) {
	if (typeof target[_iteratorSymbol] === "function") {
		var iterator = target[_iteratorSymbol](), step, pact, reject;
		function _cycle(result) {
			try {
				while (!(step = iterator.next()).done && (!check || !check())) {
					result = body(step.value);
					if (result && result.then) {
						if (_isSettledPact(result)) {
							result = result.v;
						} else {
							result.then(_cycle, reject || (reject = _settle.bind(null, pact = new _Pact(), 2)));
							return;
						}
					}
				}
				if (pact) {
					_settle(pact, 1, result);
				} else {
					pact = result;
				}
			} catch (e) {
				_settle(pact || (pact = new _Pact()), 2, e);
			}
		}
		_cycle();
		if (iterator.return) {
			var _fixup = function(value) {
				try {
					if (!step.done) {
						iterator.return();
					}
				} catch(e) {
				}
				return value;
			};
			if (pact && pact.then) {
				return pact.then(_fixup, function(e) {
					throw _fixup(e);
				});
			}
			_fixup();
		}
		return pact;
	}
	// No support for Symbol.iterator
	if (!("length" in target)) {
		throw new TypeError("Object is not iterable");
	}
	// Handle live collections properly
	var values = [];
	for (var i = 0; i < target.length; i++) {
		values.push(target[i]);
	}
	return _forTo(values, function(i) { return body(values[i]); }, check);
}

const _asyncIteratorSymbol = /*#__PURE__*/ typeof Symbol !== "undefined" ? (Symbol.asyncIterator || (Symbol.asyncIterator = Symbol("Symbol.asyncIterator"))) : "@@asyncIterator";

// Asynchronously implement a generic for loop
function _for(test, update, body) {
	var stage;
	for (;;) {
		var shouldContinue = test();
		if (_isSettledPact(shouldContinue)) {
			shouldContinue = shouldContinue.v;
		}
		if (!shouldContinue) {
			return result;
		}
		if (shouldContinue.then) {
			stage = 0;
			break;
		}
		var result = body();
		if (result && result.then) {
			if (_isSettledPact(result)) {
				result = result.s;
			} else {
				stage = 1;
				break;
			}
		}
		if (update) {
			var updateValue = update();
			if (updateValue && updateValue.then && !_isSettledPact(updateValue)) {
				stage = 2;
				break;
			}
		}
	}
	var pact = new _Pact();
	var reject = _settle.bind(null, pact, 2);
	(stage === 0 ? shouldContinue.then(_resumeAfterTest) : stage === 1 ? result.then(_resumeAfterBody) : updateValue.then(_resumeAfterUpdate)).then(void 0, reject);
	return pact;
	function _resumeAfterBody(value) {
		result = value;
		do {
			if (update) {
				updateValue = update();
				if (updateValue && updateValue.then && !_isSettledPact(updateValue)) {
					updateValue.then(_resumeAfterUpdate).then(void 0, reject);
					return;
				}
			}
			shouldContinue = test();
			if (!shouldContinue || (_isSettledPact(shouldContinue) && !shouldContinue.v)) {
				_settle(pact, 1, result);
				return;
			}
			if (shouldContinue.then) {
				shouldContinue.then(_resumeAfterTest).then(void 0, reject);
				return;
			}
			result = body();
			if (_isSettledPact(result)) {
				result = result.v;
			}
		} while (!result || !result.then);
		result.then(_resumeAfterBody).then(void 0, reject);
	}
	function _resumeAfterTest(shouldContinue) {
		if (shouldContinue) {
			result = body();
			if (result && result.then) {
				result.then(_resumeAfterBody).then(void 0, reject);
			} else {
				_resumeAfterBody(result);
			}
		} else {
			_settle(pact, 1, result);
		}
	}
	function _resumeAfterUpdate() {
		if (shouldContinue = test()) {
			if (shouldContinue.then) {
				shouldContinue.then(_resumeAfterTest).then(void 0, reject);
			} else {
				_resumeAfterTest(shouldContinue);
			}
		} else {
			_settle(pact, 1, result);
		}
	}
}

var isDataNode = function isDataNode(node) {
  if (!node) {
    return false;
  }

  return "id" in node;
};

var findNodeIndex = function findNodeIndex(parentNode, node) {
  var children = parentNode.children;

  if (!children || typeof children === "function" || !children.length) {
    return null;
  }

  var index = children.findIndex(function (n) {
    return node.id === n.id;
  });

  if (typeof index !== "number") {
    return null;
  }

  return index;
};

var findNodeIndexOrThrowError = function findNodeIndexOrThrowError(parentNode, node) {
  var index = parentNode ? findNodeIndex(parentNode, node) : null;

  if (index === null) {
    throw new Error("Couldn\t retrieve start or end index. Provide either start and end as numbers or either parent node has to have an array of children (not empty)");
  }

  return index;
};

var useFetch = function useFetch(addJob, loader, setTreeData) {
  var initiatedRoot = useRef(false);
  var queueRef = useRef(new Map());
  var dragOperationsForbiddenRef = useRef(false);

  var isFetchingForParent = function isFetchingForParent(parentNode) {
    var queue = queueRef.current.get(parentNode ? parentNode.id : parentNode);
    return Boolean(queue && queue.length);
  };

  var isFetching = function isFetching(parentNode, node) {
    var _node$data$parent;

    var parentId = node !== null && node !== void 0 && (_node$data$parent = node.data.parent) !== null && _node$data$parent !== void 0 && _node$data$parent.id ? node.data.parent.id : null;
    var nodeIndex = -1;
    var idx = findNodeIndex(parentNode, node);

    if (idx === null) {
      throw new Error("Item not found!");
    }

    nodeIndex = idx;
    var loading = queueRef.current.get(parentId);

    if (!loading || !loading.length) {
      return false;
    }

    for (var _iterator = _createForOfIteratorHelperLoose(loading), _step; !(_step = _iterator()).done;) {
      var indices = _step.value;

      if (nodeIndex >= indices[1] && nodeIndex <= indices[2]) {
        return true;
      }
    }

    return false;
  };

  var flush = function flush(parentNode, startIndex, endIndex) {
    try {
      return Promise.resolve(loader(parentNode, startIndex, endIndex)).then(function (newNodes) {
        return addJob(function () {
          var error = null;
          setTreeData(function (oldTree) {
            dragOperationsForbiddenRef.current = true;
            var tree = oldTree;

            try {
              tree = findParentAndAppendNodes({
                parentId: parentNode ? parentNode.id : null,
                treeData: oldTree,
                nodesToAppend: newNodes,
                replace: {
                  start: startIndex
                }
              });
            } catch (err) {
              error = err;
              console.error(error);
            }

            dragOperationsForbiddenRef.current = false;
            return tree;
          });
          return {
            error: error,
            newNodes: newNodes
          };
        });
      });
    } catch (e) {
      return Promise.reject(e);
    }
  };

  var fetchNodes = function fetchNodes(meta, childrenCount, refetch, onlyInitial) {
    try {
      var _meta$parentNode, _loading2;

      var _temp6 = function _temp6() {
        var _queueRef$current, _queueRef$current$ent;

        if (onlyInitial && initiated) {
          var _loading;

          (_loading = loading) === null || _loading === void 0 ? void 0 : _loading.shift();
          return;
        }

        if (!emptyQueue) {
          return;
        }

        var queueEntries = (_queueRef$current = queueRef.current) === null || _queueRef$current === void 0 ? void 0 : (_queueRef$current$ent = _queueRef$current.entries) === null || _queueRef$current$ent === void 0 ? void 0 : _queueRef$current$ent.call(_queueRef$current);
        return _forOf(queueEntries, function (val) {
          var loading = val[1];

          var _temp2 = function () {
            if (loading && loading.length) {
              var _temp5 = _for(function () {
                return !!loading.length;
              }, void 0, function () {
                var indices = loading.shift();

                if (!indices) {
                  return;
                }

                return Promise.resolve(flush.apply(void 0, indices)).then(function () {});
              });

              if (_temp5 && _temp5.then) return _temp5.then(function () {});
            }
          }();

          if (_temp2 && _temp2.then) return _temp2.then(function () {});
        });
      };

      var startIndex = 0;
      var endIndex = 0;

      if (typeof meta.startIndex === "number") {
        startIndex = meta.startIndex;
      } else {
        startIndex = findNodeIndexOrThrowError(meta.parentNode, meta.startIndex);
      }

      if (typeof meta.endIndex === "number") {
        endIndex = meta.endIndex;
      } else {
        endIndex = findNodeIndexOrThrowError(meta.parentNode, meta.endIndex);
      }

      var parentId = isDataNode(meta.parentNode) ? ((_meta$parentNode = meta.parentNode) === null || _meta$parentNode === void 0 ? void 0 : _meta$parentNode.id) || null : null;
      var emptyQueue = true;
      var loading = queueRef.current.get(parentId);
      var parentNode = isDataNode(meta.parentNode) ? meta.parentNode : null;

      if ((_loading2 = loading) !== null && _loading2 !== void 0 && _loading2.length) {
        loading.push([parentNode, startIndex, endIndex]);
        emptyQueue = false;
      } else {
        queueRef.current.set(parentId, [[parentNode, startIndex, endIndex]]);
        loading = queueRef.current.get(parentId);
      }

      var initiated = !refetch && (!isDataNode(meta.parentNode) ? initiatedRoot.current : meta.parentNode.initiated);

      var _temp7 = function () {
        if (!initiated) {
          if (!parentNode) {
            initiatedRoot.current = true;
          }

          dragOperationsForbiddenRef.current = true;
          return Promise.resolve(buildAndAppendSkeletons({
            parent: parentNode,
            count: childrenCount,
            addJob: addJob,
            setTreeData: setTreeData,
            append: "instead",
            parentData: {
              initiated: true
            }
          })).then(function () {
            dragOperationsForbiddenRef.current = false;
          });
        }
      }();

      return Promise.resolve(_temp7 && _temp7.then ? _temp7.then(_temp6) : _temp6(_temp7));
    } catch (e) {
      return Promise.reject(e);
    }
  };

  return {
    fetchNodes: fetchNodes,
    isFetching: isFetching,
    isFetchingForParent: isFetchingForParent,
    dragOperationsForbiddenRef: dragOperationsForbiddenRef
  };
};

var styles$2 = {"root":"_3cTpQ"};

var debounce = require("lodash.debounce");

var useScrollQueue = function useScrollQueue(rootRef) {
  var _useState = useState(false),
      scrolling = _useState[0],
      setScrolling = _useState[1];

  var scrollingRef = useRef(scrolling);
  var jobsRef = useRef([]);
  useEffect(function () {
    var root = rootRef.current;
    var scrollableContainer = null;
    var removeScrolling = debounce(function () {
      setScrolling(false);
    }, 1000);

    var scroll = function scroll() {
      setScrolling(true);
      removeScrolling();
    };

    (function () {
      try {
        return function () {
          if (root) {
            return Promise.resolve(new Promise(function (resolve) {
              setTimeout(function () {
                resolve(null);
              }, 1500);
            })).then(function () {
              scrollableContainer = root.querySelector(".rst__virtualScrollOverride");

              if (!scrollableContainer) {
                return;
              }

              scrollableContainer.addEventListener("scroll", scroll);
            });
          }
        }();
      } catch (e) {
        Promise.reject(e);
      }
    })();

    return function () {
      if (scrollableContainer) {
        scrollableContainer.removeEventListener("scroll", scroll);
      }
    };
  }, []);

  var jobsHandler = function jobsHandler() {
    if (scrollingRef.current) {
      return;
    }

    var jobs = jobsRef.current;

    while (jobs.length) {
      var j = jobs.shift();

      if (j) {
        var result = j.job();
        j.resolve(result);
      }
    }
  };

  useEffect(function () {
    scrollingRef.current = scrolling;
    jobsHandler();
  }, [scrolling]);

  var addJob = function addJob(job) {
    var promise = new Promise(function (resolve) {
      try {
        jobsRef.current.push({
          resolve: resolve,
          job: job
        });
        jobsHandler();
        return Promise.resolve();
      } catch (e) {
        return Promise.reject(e);
      }
    });
    return promise;
  };

  return {
    addJob: addJob
  };
};

var _excluded$2 = ["onDragStateChanged", "onNodeSelected", "onDraggedFinished", "onTreeDataChange", "url", "loader"];

var debounce$1 = require("lodash.debounce");

var VirtualTreeImpl = function VirtualTreeImpl(props, ref) {
  var rootItems = props.countOfRootItems || 1000;

  var onDragStateChangedFromProps = props.onDragStateChanged,
      onNodeSelected = props.onNodeSelected,
      onDraggedFinished = props.onDraggedFinished,
      onTreeDataChange = props.onTreeDataChange,
      url = props.url,
      userLoader = props.loader,
      rest = _objectWithoutPropertiesLoose(props, _excluded$2);

  var toggleNode = useCallback(function (props) {
    var path = props.path,
        expanded = props.expanded,
        treeData = props.treeData;
    var data = getNodeAtPath({
      treeData: treeData,
      path: path,
      getNodeKey: getNodeKey
    });

    if (data) {
      var node = data.node;
      var newTD = changeNodeAtPath({
        treeData: treeData,
        path: path,
        getNodeKey: getNodeKey,
        newNode: _extends({}, node, {
          expanded: expanded !== undefined ? expanded : !node.expanded
        })
      });
      var newData = getNodeAtPath({
        treeData: newTD,
        path: path,
        getNodeKey: getNodeKey
      });
      setTreeData(function () {
        return newTD;
      });
      onVisibilityToggle({
        treeData: newTD,
        node: newData.node,
        expanded: !!newData.node.expanded
      });
    }
  }, []);
  var iRef = useRef(null);
  var forceLoadCurrentSkeletons = useCallback(function () {
    var _iRef$current;

    (_iRef$current = iRef.current) === null || _iRef$current === void 0 ? void 0 : _iRef$current.resetloadMoreItemsCache(true);
  }, []);

  var _useState = useState(props.initalTreeData || []),
      treeData = _useState[0],
      setTreeData = _useState[1];

  useEffect(function () {
    onTreeDataChange && onTreeDataChange(treeData);
  }, [treeData]);

  var _onDragStateChanged = useOnDragFinished(treeData, onDraggedFinished);

  var _useNodeSelection = useNodeSelection(onNodeSelected),
      _generateNodeProps = _useNodeSelection.generateNodeProps;

  var loader = useMemo(function () {
    if (userLoader) {
      return userLoader;
    }

    var loadNodes = createLoadNodes(url);
    return createLoader(loadNodes, makeDefaultDataNode);
  }, []);
  var rootRef = useRef(null);

  var _useScrollQueue = useScrollQueue(rootRef),
      addJob = _useScrollQueue.addJob;

  var _useFetch = useFetch(addJob, loader, setTreeData),
      fetchNodes = _useFetch.fetchNodes,
      isFetching = _useFetch.isFetching,
      dragOperationsForbiddenRef = _useFetch.dragOperationsForbiddenRef;

  useImperativeHandle(ref, function () {
    return {
      toggleNode: toggleNode,
      setTreeData: setTreeData,
      treeData: treeData,
      fetchNodes: fetchNodes,
      forceLoadCurrentSkeletons: forceLoadCurrentSkeletons
    };
  }, [toggleNode, treeData, fetchNodes]);
  useEffect(function () {
    if (!props.initalTreeData) {
      try {
        fetchNodes({
          parentNode: null,
          startIndex: 0,
          endIndex: 10
        }, rootItems);
      } catch (e) {
        Promise.reject(e);
      }
    }
  }, []);

  var _useState2 = useState([]),
      flatData = _useState2[0],
      setFlatData = _useState2[1];

  useEffect(function () {
    setFlatData(getFlatDataFromTree({
      treeData: treeData,
      getNodeKey: getNodeKey,
      ignoreCollapsed: true
    }));
  }, [treeData]);
  var onVisibilityToggle = createOnVisibilityToggle(fetchNodes);
  var loadMoreItems = useMemo(function () {
    return debounce$1(function (startIndex, endIndex) {
      try {
        var sliceFlat = flatData.slice(startIndex, endIndex + 1);

        if (!sliceFlat.length) {
          return Promise.resolve();
        }

        var fethConfigMap = sliceFlat.reduce(function (acc, currentNode) {
          var parentNode = currentNode.parentNode || null;
          var parentId = parentNode ? parentNode.id : null;
          var entry = acc.get(parentId);
          var node = currentNode.node;

          if (entry) {
            entry.meta.endIndex = node;
          } else {
            var newEntry = {
              meta: {
                parentNode: parentNode === null ? {
                  children: treeData
                } : parentNode,
                startIndex: node,
                endIndex: node
              },
              childrenCount: parentNode ? parentNode.childrenCount : rootItems
            };
            acc.set(parentId, newEntry);
          }

          return acc;
        }, new Map());

        for (var _iterator = _createForOfIteratorHelperLoose(fethConfigMap), _step; !(_step = _iterator()).done;) {
          var entry = _step.value;
          var data = entry[1];
          fetchNodes(data.meta, data.childrenCount);
        }

        return Promise.resolve();
      } catch (e) {
        return Promise.reject(e);
      }
    }, 250);
  }, [flatData, treeData]);
  var theme = useMemo(function () {
    return createFileExplorerTheme(props.renderers);
  }, []);
  return React.createElement("div", {
    ref: rootRef,
    className: styles$2.root
  }, React.createElement(InfiniteLoader, {
    ref: iRef,
    isItemLoaded: function isItemLoaded(index) {
      var flatItem = flatData[index];

      if (!flatItem) {
        return true;
      }

      var parentNode = flatItem.parentNode;
      var node = flatItem.node ? flatItem.node : null;

      if (!node) {
        throw new Error("Node doesn't exist");
      }

      if (!isPlaceholder(flatItem.node)) {
        return true;
      }

      return isFetching(parentNode ? parentNode : {
        children: treeData
      }, node);
    },
    loadMoreItems: loadMoreItems,
    itemCount: Infinity
  }, function (_ref) {
    var onItemsRendered = _ref.onItemsRendered;
    return React.createElement(SortableTree, Object.assign({
      treeData: treeData,
      onChange: function onChange(tData) {
        return setTreeData(tData);
      },
      canDrag: function canDrag(data) {
        var interceptorAllows = true;

        if (props.canDragInterceptor) {
          interceptorAllows = props.canDragInterceptor(data);
        }

        if (!interceptorAllows) {
          return false;
        }

        return createCannotDragSkeleton(dragOperationsForbiddenRef)(data);
      },
      canDrop: function canDrop(data) {
        var interceptorAllows = true;

        if (props.canDropInterceptor) {
          interceptorAllows = props.canDropInterceptor(data);
        }

        if (!interceptorAllows) {
          return false;
        }

        return cannotDropSkeleton(data);
      },
      canNodeHaveChildren: skeletonCannotHaveChildren,
      reactVirtualizedListProps: {
        onRowsRendered: function onRowsRendered(info) {
          return onItemsRendered({
            overscanStartIndex: info.overscanStartIndex,
            overscanStopIndex: info.overscanStopIndex,
            visibleStartIndex: info.startIndex,
            visibleStopIndex: info.stopIndex
          });
        }
      },
      theme: theme,
      getNodeKey: getNodeKey,
      isVirtualized: true,
      generateNodeProps: function generateNodeProps(rowInfo) {
        var node = rowInfo.node,
            path = rowInfo.path;
        return Object.assign({
          "data-id": "id" + node.id,
          path: path
        }, _generateNodeProps(rowInfo));
      },
      onDragStateChanged: function onDragStateChanged() {
        _onDragStateChanged.apply(void 0, arguments);

        if (onDragStateChangedFromProps) {
          onDragStateChangedFromProps.apply(void 0, arguments);
        }
      },
      onVisibilityToggle: onVisibilityToggle,
      scaffoldBlockPxWidth: 28,
      rowHeight: 40
    }, rest));
  }));
};

var VirtualTree = React.forwardRef(VirtualTreeImpl);

export { LOADING_BATCH_COUNT, PLACEHOLDER_LOADING_TYPE, PLACEHOLDER_SKELETON_TYPE, PLACEHOLDER_TYPE, VirtualTree, addNodesUnderParent, cannotDropSkeleton, createCannotDragSkeleton, createLoader, findParentAndAppendNodes, findParentNode, getNodeKey, skeletonCannotHaveChildren, useScrollQueue };
//# sourceMappingURL=index.modern.js.map
